@@ -1,4 +1,4 @@
-require 'securecompare'
+require 'active_support/security_utils'
 
 module JSON
   module JOSE
@@ -6,11 +6,10 @@ module JSON
 
     included do
       extend ClassMethods
-      include SecureCompare
       register_header_keys :alg, :jku, :jwk, :x5u, :x5t, :x5c, :kid, :typ, :cty, :crit
       alias_method :algorithm, :alg
 
-      attr_accessor :header
+      attr_writer :header
       def header
         @header ||= {}
       end
@@ -27,12 +26,24 @@ module JSON
       when JSON::JWK::Set
         key.detect do |jwk|
           jwk[:kid] && jwk[:kid] == kid
-        end.try(:to_key) or raise JWK::Set::KidNotFound
+        end&.to_key or raise JWK::Set::KidNotFound
       else
         key
       end
     end
 
+    def secure_compare(a, b)
+      if ActiveSupport::SecurityUtils.respond_to?(:fixed_length_secure_compare)
+        begin
+          ActiveSupport::SecurityUtils.fixed_length_secure_compare(a, b)
+        rescue ArgumentError
+          false
+        end
+      else
+        ActiveSupport::SecurityUtils.secure_compare(a, b)
+      end
+    end
+
     module ClassMethods
       def register_header_keys(*keys)
         keys.each do |header_key|
@@ -45,15 +56,15 @@ module JSON
         end
       end
 
-      def decode(input, key_or_secret = nil)
+      def decode(input, key_or_secret = nil, algorithms = nil, encryption_methods = nil, allow_blank_payload = false)
         if input.is_a? Hash
-          decode_json_serialized input, key_or_secret
+          decode_json_serialized input, key_or_secret, algorithms, encryption_methods, allow_blank_payload
         else
-          decode_compact_serialized input, key_or_secret
+          decode_compact_serialized input, key_or_secret, algorithms, encryption_methods, allow_blank_payload
         end
-      rescue MultiJson::DecodeError
+      rescue JSON::ParserError, ArgumentError
         raise JWT::InvalidFormat.new("Invalid JSON Format")
       end
     end
   end
-end
\ No newline at end of file
+end
