@@ -14,9 +14,10 @@ module JSON
 
     attr_accessor(
       :public_key_or_secret, :private_key_or_secret,
-      :plain_text, :cipher_text, :authentication_tag, :iv, :auth_data,
-      :content_encryption_key, :jwe_encrypted_key, :encryption_key, :mac_key
+      :plain_text, :cipher_text, :iv, :auth_data,
+      :content_encryption_key, :encryption_key, :mac_key
     )
+    attr_writer :jwe_encrypted_key, :authentication_tag
 
     register_header_keys :enc, :epk, :zip, :apu, :apv
     alias_method :encryption_method, :enc
@@ -98,15 +99,15 @@ module JSON
     # common
 
     def gcm?
-      [:A128GCM, :A256GCM].include? encryption_method.try(:to_sym)
+      [:A128GCM, :A256GCM].include? encryption_method&.to_sym
     end
 
     def cbc?
-      [:'A128CBC-HS256', :'A256CBC-HS512'].include? encryption_method.try(:to_sym)
+      [:'A128CBC-HS256', :'A256CBC-HS512'].include? encryption_method&.to_sym
     end
 
     def dir?
-      :dir == algorithm.try(:to_sym)
+      :dir == algorithm&.to_sym
     end
 
     def cipher
@@ -115,7 +116,7 @@ module JSON
     end
 
     def cipher_name
-      case encryption_method.try(:to_sym)
+      case encryption_method&.to_sym
       when :A128GCM
         'aes-128-gcm'
       when :A256GCM
@@ -130,7 +131,7 @@ module JSON
     end
 
     def sha_size
-      case encryption_method.try(:to_sym)
+      case encryption_method&.to_sym
       when :'A128CBC-HS256'
         256
       when :'A256CBC-HS512'
@@ -158,7 +159,7 @@ module JSON
     # encryption
 
     def jwe_encrypted_key
-      @jwe_encrypted_key ||= case algorithm.try(:to_sym)
+      @jwe_encrypted_key ||= case algorithm&.to_sym
       when :RSA1_5
         public_key_or_secret.public_encrypt content_encryption_key
       when :'RSA-OAEP'
@@ -210,7 +211,7 @@ module JSON
 
     def decrypt_content_encryption_key
       fake_content_encryption_key = generate_content_encryption_key # NOTE: do this always not to make timing difference
-      case algorithm.try(:to_sym)
+      case algorithm&.to_sym
       when :RSA1_5
         private_key_or_secret.private_decrypt jwe_encrypted_key
       when :'RSA-OAEP'
@@ -248,12 +249,12 @@ module JSON
     end
 
     class << self
-      def decode_compact_serialized(input, private_key_or_secret, algorithms = nil, encryption_methods = nil)
+      def decode_compact_serialized(input, private_key_or_secret, algorithms = nil, encryption_methods = nil, _allow_blank_payload = false)
         unless input.count('.') + 1 == NUM_OF_SEGMENTS
           raise InvalidFormat.new("Invalid JWE Format. JWE should include #{NUM_OF_SEGMENTS} segments.")
         end
         jwe = new
-        _header_json_, jwe.jwe_encrypted_key, jwe.iv, jwe.cipher_text, jwe.authentication_tag = input.split('.').collect do |segment|
+        _header_json_, jwe.jwe_encrypted_key, jwe.iv, jwe.cipher_text, jwe.authentication_tag = input.split('.', NUM_OF_SEGMENTS).collect do |segment|
           begin
             Base64.urlsafe_decode64 segment
           rescue ArgumentError
@@ -268,7 +269,7 @@ module JSON
         jwe
       end
 
-      def decode_json_serialized(input, private_key_or_secret, algorithms = nil, encryption_methods = nil)
+      def decode_json_serialized(input, private_key_or_secret, algorithms = nil, encryption_methods = nil, _allow_blank_payload = false)
         input = input.with_indifferent_access
         jwe_encrypted_key = if input[:recipients].present?
           input[:recipients].first[:encrypted_key]
