@@ -7,9 +7,17 @@ describe JSON::JWS do
     _jwt_.alg = alg
     _jwt_
   end
+  let(:jwt_blank) do
+    _jwt_ = JSON::JWT.new nil
+    _jwt_.alg = alg
+    _jwt_
+  end
   let(:jws) { JSON::JWS.new jwt }
+  let(:jws_blank) { JSON::JWS.new jwt_blank }
   let(:signed) { jws.sign! private_key_or_secret }
+  let(:signed_blank) { jws_blank.sign! private_key_or_secret }
   let(:decoded) { JSON::JWT.decode signed.to_s, public_key_or_secret }
+  let(:decoded_blank) { JSON::JWT.decode signed_blank.to_s, public_key_or_secret, nil, nil, true }
   let(:claims) do
     {
       iss: 'joe',
@@ -27,6 +35,16 @@ describe JSON::JWS do
       :RS512 => 'EHeGM2Mo3ghhUfSB99AlREehrbC6OPE-nYL_rwf88ysTnJ8L1QQ0UuCrXq4SpRutGLK_bYTK3ZALvFRPoOgK_g0QWmqv6qjQRU_QTxoq8y8APP-IgKKDuIiGH6daBV2rAPLDReqYNKsKjmTvZJo2c0a0e_WZkkj_ZwpgjTG3v0gW9lbDAzLJDz18eqtR4ZO7JTu_fyNrUrNk-w2_wpxSsn9sygIMp0lKE0_pt0b01fz3gjTDjlltU0cKSalUp4geaBDH7QRcexrolIctdQFbNKTXQxoigxD3NLNkKGH7f6A8KZdcOm8AnEjullcZs8_OWGnW43p1qrxoBRSivb9pqQ'
     }
   end
+  let(:expected_signature_blank_payload) do
+    {
+      :HS256 => 'iRFMM3GknVfzRTxlVQT87jfIw32Ik3lUYNGePPk5wnM',
+      :HS384 => 'rxyzr3I2RWRBgQaewQt3yjdp3BqkrFh-iHcet318OYHWhXvyzAE0npf0l0xi5DOV',
+      :HS512 => 'VDHOrPYrwycjaKbwccObXi6dmw4fVFqiFsNFQjqYHQAkxJGxqhfVLc1_WfKMa6C7vGSGroabaVdK7nn08XPdSQ',
+      :RS256 => 'WthQjouPVbErM7McwSY4slJjHaWqmFg1qKdmTDvttkiyAEcTjVViJkNHH9Mp573h13cXtLob1xh3UJYh5_-hSA4Y24zdyck3jp3fsOusflp1cMmhWXZ2nETKeWCEJDKRAnWynHqkwes7tgWmS0gVeuljeNkuovJlHmNRcoMR9Z3ZuiHfc2WFh-iFbM5Zne1y-_SSgAZwOD20P0Ysn28DtJTlXcm74ENqhLEJnvHS-872d6surb23kHMns43GtT5bm-aJoMLct0nO1GBapQAiKUknTsw24IfOkX4vJNQzIWVSzx3zOxXjcVHlH92af6NknIlPCfRparLC9YEK2NkJYg',
+      :RS384 => 'Jy6XNLNAyujRHYoCOtFqu7z0imHZMiwkwBr73ok_DDSDxQSA9ryt_q_tX0u8knpAIRcTJuNA0-s5DkGbpIj9coKgZ5JBvE_n9ijvNubImf8_vCDDitJemzUtnJypb9GbP4A3nWDAZC0KONVqlxpy92-9xrG5sFEzaYCFYZYnXv8kmmQEIVI1GXw4_Fx8HxRu5cae9WWTgaKQOFG54S303C0H966C1o6d9o3HQH7x8GEl632qBw4LzONWr_QpCN-UFgmJHO7yBwaP-RWnLDW3hYlb4IybRIvMQQicjkjNaNwLTmwo31orVxO53GcSjyhU2y_R843nQcNjTT_lD1QRvg',
+      :RS512 => 'ws2HZ6wvh8GMrFKiIHXDogyx8HFpa4wvrLxfZaMfCoMPf0SZ4V3tiEZRWfrxyvwpsdBj2Mgm5lt3IYAHhlI2hqWvuikDq6tuViloaAIm2xwTU060bF0GL1tQJ-h20wUukJ6fsWet8M9DNg7hcElYQMawHhk4L91YUtY2hKT_uWgPih_pn0Hq5Ve0at4CwAyXXTwCYSEH23PMsUdDfE5tfCyvL2bNQ71Ld_MvQS1NLS7hydzEtfxLK-UkDQVclFmEM3JXrPG7YSRodtKlwJ-ESDx6CaJXXDAgitSF32dslcIkmOXRJqjNmF15i_aVg0ExiU92WTpCrdwzWTt4Aphqlw',
+    }
+  end
 
   shared_examples_for :jwt_with_alg do
     it { should == jwt }
@@ -45,11 +63,37 @@ describe JSON::JWS do
     end
   end
 
+  describe 'decode' do
+    let(:alg) { :RS256 }
+    let(:private_key_or_secret) { private_key }
+    let(:public_key_or_secret) { public_key }
+
+    describe 'blank payload not allowed' do
+      it 'should raise format error' do
+        expect do
+          JSON::JWT.decode signed_blank.to_s, public_key_or_secret
+        end.to raise_error JSON::JWT::InvalidFormat
+      end
+    end
+    describe 'blank payload allowed' do
+      it 'should not raise an error' do
+        expect do
+          JSON::JWT.decode signed_blank.to_s, public_key_or_secret, nil, nil, true
+        end.to_not raise_error
+      end
+    end
+  end
+
   describe '#sign!' do
     shared_examples_for :generate_expected_signature do
       it do
         Base64.urlsafe_encode64(signed.signature, padding: false).should == expected_signature[alg]
       end
+      context 'with blank payload' do
+        it do
+          Base64.urlsafe_encode64(signed_blank.signature, padding: false).should == expected_signature_blank_payload[alg]
+        end
+      end
     end
     subject { signed }
 
@@ -176,6 +220,32 @@ describe JSON::JWS do
           decoded[:'http://example.com/is_root'] == true
         end
       end
+
+      context 'with blank payload' do
+        it do
+          expect { decoded_blank }.not_to raise_error
+          decoded_blank.should be_a JSON::JWT
+        end
+
+        describe 'header' do
+          let(:header) { decoded_blank.header }
+          it 'should be parsed successfully' do
+            header[:typ].should == 'JWT'
+            header[:alg].should == alg.to_s
+          end
+        end
+
+        describe 'claims' do
+          it 'should be parsed successfully' do
+            p decoded_blank.blank_payload
+            decoded_blank.blank_payload.should == true
+            decoded_blank[:iss].should == nil
+            decoded_blank[:exp].should == nil
+            decoded[:'http://example.com/is_root'] == nil
+          end
+        end
+      end
+
     end
     subject { decoded }
 
@@ -274,6 +344,12 @@ describe JSON::JWS do
         jws.to_json.should == claims.to_json
       end
     end
+    context 'with blank payload' do
+      it 'should JSONize payload' do
+        puts ("jws_blank: #{jws_blank.to_json.inspect}")
+        jws_blank.to_json.should == ''
+      end
+    end
 
     context 'when syntax option given' do
       context 'when general' do
@@ -286,6 +362,17 @@ describe JSON::JWS do
             }]
           }.to_json
         end
+        context 'with blank payload' do
+          it 'should return General JWS JSON Serialization' do
+            signed_blank.to_json(syntax: :general).should == {
+              payload: '',
+              signatures: [{
+                protected: Base64.urlsafe_encode64(signed_blank.header.to_json, padding: false),
+                signature: Base64.urlsafe_encode64(signed_blank.signature, padding: false)
+              }]
+            }.to_json
+          end
+        end
 
         context 'when not signed yet' do
           it 'should not fail' do
@@ -297,6 +384,17 @@ describe JSON::JWS do
               }]
             }.to_json
           end
+          context 'with blank payload' do
+            it 'should not fail' do
+              jws_blank.to_json(syntax: :general).should == {
+                payload: '',
+                signatures: [{
+                  protected: Base64.urlsafe_encode64(jws_blank.header.to_json, padding: false),
+                  signature: Base64.urlsafe_encode64('', padding: false)
+                }]
+              }.to_json
+            end
+          end
         end
       end
 
@@ -308,6 +406,15 @@ describe JSON::JWS do
             signature: Base64.urlsafe_encode64(signed.signature, padding: false)
           }.to_json
         end
+        context 'with blank payload' do
+          it 'should return Flattened JWS JSON Serialization' do
+            signed_blank.to_json(syntax: :flattened).should == {
+              protected: Base64.urlsafe_encode64(signed_blank.header.to_json, padding: false),
+              payload: '',
+              signature: Base64.urlsafe_encode64(signed_blank.signature, padding: false)
+            }.to_json
+          end
+        end
 
         context 'when not signed yet' do
           it 'should not fail' do
@@ -317,6 +424,15 @@ describe JSON::JWS do
               signature: Base64.urlsafe_encode64('', padding: false)
             }.to_json
           end
+          context 'with blank payload' do
+            it 'should not fail' do
+              jws_blank.to_json(syntax: :flattened).should == {
+                protected: Base64.urlsafe_encode64(jws_blank.header.to_json, padding: false),
+                payload: '',
+                signature: Base64.urlsafe_encode64('', padding: false)
+              }.to_json
+            end
+          end
         end
       end
     end
