@@ -56,10 +56,54 @@ describe JSON::JWT do
         its(:alg) { should == :HS256 }
       end
 
-      context 'otherwise' do
+      context 'when key is RSA key' do
         let(:key) { private_key }
         its(:alg) { should == :RS256 }
       end
+
+      context 'when key is EC key' do
+        context 'when prime256v1' do
+          let(:key) { private_key(:ecdsa) }
+          its(:alg) { should == :ES256 }
+        end
+
+        context 'when secp384r1' do
+          let(:key) { private_key(:ecdsa, digest_length: 384) }
+          its(:alg) { should == :ES384 }
+        end
+
+        context 'when secp521r1' do
+          let(:key) { private_key(:ecdsa, digest_length: 512) }
+          its(:alg) { should == :ES512 }
+        end
+      end
+
+      context 'when key is JWK with kty=okt' do
+        let(:key) { JSON::JWK.new shared_secret }
+        its(:alg) { should == :HS256 }
+      end
+
+      context 'when key is JWK with kty=RSA' do
+        let(:key) { JSON::JWK.new private_key }
+        its(:alg) { should == :RS256 }
+      end
+
+      context 'when key is JWK with kty=EC' do
+        context 'when prime256v1' do
+          let(:key) { JSON::JWK.new private_key(:ecdsa) }
+          its(:alg) { should == :ES256 }
+        end
+
+        context 'when secp384r1' do
+          let(:key) { JSON::JWK.new private_key(:ecdsa, digest_length: 384) }
+          its(:alg) { should == :ES384 }
+        end
+
+        context 'when secp521r1' do
+          let(:key) { JSON::JWK.new private_key(:ecdsa, digest_length: 512) }
+          its(:alg) { should == :ES512 }
+        end
+      end
     end
 
     context 'when non-JWK key is given' do
@@ -185,13 +229,13 @@ describe JSON::JWT do
               header, payload, signature = jws.to_s.split('.')
               malformed_header = {alg: :none}.to_json
               [
-                UrlSafeBase64.encode64(malformed_header),
+                Base64.urlsafe_encode64(malformed_header, padding: false),
                 payload,
                 ''
               ].join('.')
             end
 
-            it 'should do verification' do
+            it do
               expect do
                 JSON::JWT.decode malformed_jwt_string, 'secret'
               end.to raise_error JSON::JWT::VerificationFailed
@@ -209,13 +253,13 @@ describe JSON::JWT do
               header, payload, signature = jws.to_s.split('.')
               malformed_header = {alg: :none}.to_json
               [
-                UrlSafeBase64.encode64(malformed_header),
+                Base64.urlsafe_encode64(malformed_header, padding: false),
                 payload,
                 ''
               ].join('.')
             end
 
-            it 'should fail verification' do
+            it do
               expect do
                 JSON::JWT.decode malformed_jwt_string, public_key
               end.to raise_error JSON::JWT::UnexpectedAlgorithm
@@ -229,22 +273,102 @@ describe JSON::JWT do
               malformed_signature = OpenSSL::HMAC.digest(
                 OpenSSL::Digest.new('SHA256'),
                 public_key.to_s,
-                [malformed_header, payload].join('.')
+                [Base64.urlsafe_encode64(malformed_header, padding: false), payload].join('.')
               )
               [
-                UrlSafeBase64.encode64(malformed_header),
+                Base64.urlsafe_encode64(malformed_header, padding: false),
                 payload,
-                UrlSafeBase64.encode64(malformed_signature)
+                Base64.urlsafe_encode64(malformed_signature, padding: false)
               ].join('.')
             end
 
-            it 'should fail verification' do
+            it do
               expect do
                 JSON::JWT.decode malformed_jwt_string, public_key
               end.to raise_error JSON::JWS::UnexpectedAlgorithm
             end
           end
         end
+
+        context 'from alg=PS512' do
+          let(:jws) do
+            jwt.sign private_key, :PS512
+          end
+
+          if pss_supported?
+            context 'to alg=PS256' do
+              let(:malformed_jwt_string) do
+                header, payload, signature = jws.to_s.split('.')
+                malformed_header = {alg: :PS256}.to_json
+                digest = OpenSSL::Digest.new('SHA256')
+                malformed_signature = private_key.sign_pss(
+                  digest,
+                  [Base64.urlsafe_encode64(malformed_header, padding: false), payload].join('.'),
+                  salt_length: :digest,
+                  mgf1_hash: digest
+                )
+                [
+                  Base64.urlsafe_encode64(malformed_header, padding: false),
+                  payload,
+                  Base64.urlsafe_encode64(malformed_signature, padding: false)
+                ].join('.')
+              end
+
+              context 'when verification algorithm is specified' do
+                it do
+                  expect do
+                    JSON::JWT.decode malformed_jwt_string, public_key, :PS512
+                  end.to raise_error JSON::JWS::UnexpectedAlgorithm, 'Unexpected alg header'
+                end
+              end
+
+              context 'otherwise' do
+                it do
+                  expect do
+                    JSON::JWT.decode malformed_jwt_string, public_key
+                  end.not_to raise_error
+                end
+              end
+            end
+
+            context 'to alg=RS516' do
+              let(:malformed_jwt_string) do
+                header, payload, signature = jws.to_s.split('.')
+                malformed_header = {alg: :RS512}.to_json
+                malformed_signature = private_key.sign(
+                  OpenSSL::Digest.new('SHA512'),
+                  [Base64.urlsafe_encode64(malformed_header, padding: false), payload].join('.')
+                )
+                [
+                  Base64.urlsafe_encode64(malformed_header, padding: false),
+                  payload,
+                  Base64.urlsafe_encode64(malformed_signature, padding: false)
+                ].join('.')
+              end
+
+              context 'when verification algorithm is specified' do
+                it do
+                  expect do
+                    JSON::JWT.decode malformed_jwt_string, public_key, :PS512
+                  end.to raise_error JSON::JWS::UnexpectedAlgorithm, 'Unexpected alg header'
+                end
+              end
+
+              context 'otherwise' do
+                it do
+                  expect do
+                    JSON::JWT.decode malformed_jwt_string, public_key
+                  end.not_to raise_error
+                end
+              end
+            end
+          else
+            skip 'RSA PSS not supported'
+            it do
+              expect { jws }.to raise_error 'PS512 isn\'t supported. OpenSSL gem v2.1.0+ is required to use PS512.'
+            end
+          end
+        end
       end
 
       context 'when :skip_verification given as secret/key' do
@@ -280,10 +404,10 @@ describe JSON::JWT do
         context 'when general' do
           let(:serialized) do
             {
-              payload: UrlSafeBase64.encode64(claims.to_json),
+              payload: Base64.urlsafe_encode64(claims.to_json, padding: false),
               signatures: [{
-                protected: UrlSafeBase64.encode64(signed.header.to_json),
-                signature: UrlSafeBase64.encode64(signed.signature)
+                protected: Base64.urlsafe_encode64(signed.header.to_json, padding: false),
+                signature: Base64.urlsafe_encode64(signed.signature, padding: false)
               }]
             }
           end
@@ -293,9 +417,9 @@ describe JSON::JWT do
         context 'when flattened' do
           let(:serialized) do
             {
-              protected: UrlSafeBase64.encode64(signed.header.to_json),
-              payload: UrlSafeBase64.encode64(claims.to_json),
-              signature: UrlSafeBase64.encode64(signed.signature)
+              protected: Base64.urlsafe_encode64(signed.header.to_json, padding: false),
+              payload: Base64.urlsafe_encode64(claims.to_json, padding: false),
+              signature: Base64.urlsafe_encode64(signed.signature, padding: false)
             }
           end
           it_behaves_like :json_serialization_parser
@@ -320,6 +444,32 @@ describe JSON::JWT do
           end.not_to raise_error
         end
       end
+
+      context 'when alg & enc is specified' do
+        context 'when expected' do
+          it do
+            expect do
+              JSON::JWT.decode(input, private_key, 'RSA1_5', 'A128CBC-HS256')
+            end.not_to raise_error
+          end
+        end
+
+        context 'when alg is unexpected' do
+          it do
+            expect do
+              JSON::JWT.decode(input, private_key, 'dir', 'A128CBC-HS256')
+            end.to raise_error JSON::JWE::UnexpectedAlgorithm, 'Unexpected alg header'
+          end
+        end
+
+        context 'when enc is unexpected' do
+          it do
+            expect do
+              JSON::JWT.decode(input, private_key, 'RSA1_5', 'A128GCM')
+            end.to raise_error JSON::JWE::UnexpectedAlgorithm, 'Unexpected enc header'
+          end
+        end
+      end
     end
 
     context 'when JSON parse failed' do
@@ -342,10 +492,32 @@ describe JSON::JWT do
       context 'when too many dots' do
         it do
           expect do
-            JSON::JWT.decode 'header.payload.signature.something.wrong'
+            JSON::JWT.decode 'header.payload.signature.too.many.dots'
           end.to raise_error JSON::JWT::InvalidFormat
         end
       end
     end
   end
+
+  describe '.pretty_generate' do
+    subject { JSON::JWT.pretty_generate jws.to_s }
+    its(:size) { should == 2 }
+    its(:first) do
+      should == <<~HEADER.chop
+        {
+          "typ": "JWT",
+          "alg": "HS256"
+        }
+      HEADER
+    end
+    its(:last) do
+      should == <<~HEADER.chop
+        {
+          "iss": "joe",
+          "exp": 1300819380,
+          "http://example.com/is_root": true
+        }
+      HEADER
+    end
+  end
 end
